--author: igor29381
--v1.3
--date 13.05.18

HydraulicPipe = {};
AttacherJoints.registerJointType("hydraulicPipe");


function HydraulicPipe.prerequisitesPresent(specializations)
    return true;
end

HydraulicPipe.curModDir = g_currentModDirectory;

function HydraulicPipe:load(savegame)
    if hasXMLProperty(self.xmlFile, "vehicle.hydraulicPipe") then
		self.hydraulicPipe = {};
		self.attachPipe = HydraulicPipe.attachPipe;
		self.detachPipe = HydraulicPipe.detachPipe;
		self.getHydraulicPipeAttacher = HydraulicPipe.getHydraulicPipeAttacher;
		self.getIsFoldAllowed = HydraulicPipe.getIsFoldAllowed;
		self.isDetachAllowed = Utils.overwrittenFunction(self.isDetachAllowed, HydraulicPipe.isDetachAllowed);
		self.getIsTurnedOnAllowed = HydraulicPipe.getIsTurnedOnAllowed;
		self.getDetachIsPossible = HydraulicPipe.getDetachIsPossible;
		local index = getXMLString(self.xmlFile, "vehicle.hydraulicPipe#index");
		self.hydraulicPipe.index = Utils.indexToObject(self.components, index);
		local startIndex = getXMLString(self.xmlFile, "vehicle.hydraulicPipe#startIndex");
		self.hydraulicPipe.startIndex = Utils.indexToObject(self.components, startIndex);
		local animationName = getXMLString(self.xmlFile, "vehicle.hydraulicPipe#animationName");
		if animationName and animationName ~= "" then
			self.hydraulicPipe.attacherPipeAnimation = animationName;
		end;
		self.hydraulicPipe.manualAttach = Utils.getNoNil(getXMLBool(self.xmlFile, "vehicle.hydraulicPipe#manualAttach"), false);
		self.hydraulicPipe.manualAttachActive = false;
		self.hydraulicPipe.manualAttachText = {};
		local linkHydro = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.hydraulicPipe#linkText"), "linkHydro");
		local unlinkHydro = Utils.getNoNil(getXMLString(self.xmlFile, "vehicle.hydraulicPipe#unlinkText"), "unlinkHydro");
		self.hydraulicPipe.manualAttachText[true] = g_i18n:getText(unlinkHydro);
		self.hydraulicPipe.manualAttachText[false] = g_i18n:getText(linkHydro);
		local linkInputButton = getXMLString(self.xmlFile, "vehicle.hydraulicPipe#linkInputButton");
		if linkInputButton then
			self.hydraulicPipe.linkInputButton = InputBinding[linkInputButton];
		end
		self.hydraulicPipe.linkInputButton = Utils.getNoNil(self.hydraulicPipe.linkInputButton, InputBinding.ATTACH);
		self.hydraulicPipe.hasHydraulicPipeAttacher = false;
	end;
end;

function HydraulicPipe:postLoad(savegame)
	if savegame then
		self.hydraulicPipe.manualAttachActive = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key.."#manualAttachActive"), false);
		if self.hydraulicPipe.manualAttach and self.hydraulicPipe.manualAttachActive then
			self.hydraulicPipe.firstRun = true;
		end;
	end;
end;

function HydraulicPipe:delete()
end;

function HydraulicPipe:mouseEvent(posX, posY, isDown, isUp, button)
end;

function HydraulicPipe:keyEvent(unicode, sym, modifier, isDown)
end;

function HydraulicPipe:readStream(streamId, connection)
	self.hydraulicPipe.manualAttachActive = streamReadBool(streamId);
	if self.hydraulicPipe.manualAttach and self.hydraulicPipe.manualAttachActive then
		self.hydraulicPipe.firstRun = true;
	end;
end;

function HydraulicPipe:writeStream(streamId, connection)
	streamWriteBool(streamId, self.hydraulicPipe.manualAttachActive);
end;

function HydraulicPipe:getSaveAttributesAndNodes(nodeIdent)
	local attributes = ' manualAttachActive="'..tostring(self.hydraulicPipe.manualAttachActive)..'"';
    local node = nil;
    return attributes, node;
end;

function HydraulicPipe:onAttach(attacherVehicle)
	if self.hydraulicPipe then
		local attacherJointTransform = self:getHydraulicPipeAttacher(attacherVehicle);
		if attacherJointTransform then
			self.hydraulicPipe.hasHydraulicPipeAttacher = true;
			if not self.hydraulicPipe.manualAttach then
				self:attachPipe(attacherJointTransform);
			end;
		end;
	end;
end;

function HydraulicPipe:onDetach()
	self.hydraulicPipe.hasHydraulicPipeAttacher = false;
	self:detachPipe();
end;

function HydraulicPipe:isDetachAllowed()
	if self.hydraulicPipe.manualAttach then
		if self.hydraulicPipe.manualAttachActive then
			return false;
		end;
	end;
	return true;
end;

function HydraulicPipe:attachPipe(attacherJointTransform)
	for _, movingPart in pairs(self.activeDirtyMovingParts) do
		if movingPart.node == self.hydraulicPipe.index then
			movingPart.referenceFrameBackUp = movingPart.referenceFrame;
			movingPart.referenceFrame = attacherJointTransform;
			movingPart.referenceFrameOffset = {0,0,0};
			Cylindered.updateMovingPart(self, movingPart, false);
			break;
		end;
	end;
	self:playAnimation(self.hydraulicPipe.attacherPipeAnimation, 1, nil, true);
	if self.movingTools then
		for i=1, #self.movingTools do
			self.movingTools[i].isFreezed = false;
		end;
	end;
	self.hydraulicPipe.manualAttachActive = true;
end;

function HydraulicPipe:detachPipe()
	if self.hydraulicPipe.manualAttachActive then
        for _, movingPart in pairs(self.activeDirtyMovingParts) do
            if movingPart.node == self.hydraulicPipe.index then
                movingPart.referenceFrame = movingPart.referenceFrameBackUp;
                break;
            end;
        end;
		self:playAnimation(self.hydraulicPipe.attacherPipeAnimation, -1, nil, true);
		if self.movingTools then
			for i=1, #self.movingTools do
				self.movingTools[i].isFreezed = true;
			end;
		end;
		self.hydraulicPipe.manualAttachActive = false;
	end;
end;

function HydraulicPipe:getHydraulicPipeAttacher(attacherVehicle)
	for _,attacherJoint in pairs(attacherVehicle.attacherJoints) do
		if attacherJoint.jointType == AttacherJoints.JOINTTYPE_HYDRAULICPIPE then
			return attacherJoint.jointTransform;
		end;
	end;
end;

function HydraulicPipe:getIsFoldAllowed(superFunc, onAiTurnOn)
	local rootAttacherVehicle = self:getRootAttacherVehicle();
	if rootAttacherVehicle.getIsMotorStarted and rootAttacherVehicle:getIsMotorStarted() then
		if self.hydraulicPipe.manualAttach and self.hydraulicPipe.hasHydraulicPipeAttacher and not self.hydraulicPipe.manualAttachActive then
			return false;
		end;
	else
		return false;
	end;
    return true;
end;

function HydraulicPipe:getIsTurnedOnAllowed(isTurnedOn)
    local rootAttacherVehicle = self:getRootAttacherVehicle();
	if rootAttacherVehicle.getIsMotorStarted and rootAttacherVehicle:getIsMotorStarted() then
		if self.hydraulicPipe.manualAttach and self.hydraulicPipe.hasHydraulicPipeAttacher and not self.hydraulicPipe.manualAttachActive then
			return false;
		end;
	else
		return false;
	end;
    return true;
end

function HydraulicPipe:getDetachIsPossible()
	if self.hydraulicPipe.manualAttachActive then
		return not self:getIsTurnedOn();
	end;
	return true;
end;

function HydraulicPipe:update(dt)
	if self.hydraulicPipe.manualAttach then
		if self.hydraulicPipe.firstRun then
			if self.attacherVehicle then
				local attacherJointTransform = self:getHydraulicPipeAttacher(self.attacherVehicle);
				if attacherJointTransform then
					self:attachPipe(attacherJointTransform);
				end;
			else
				self.hydraulicPipe.manualAttachActive = false;
			end;
			self.hydraulicPipe.firstRun = false;
		end;
		local x,_,z = worldToLocal(self.hydraulicPipe.startIndex, getWorldTranslation(g_currentMission.player.rootNode));
		if Utils.vector2Length(x, z) < 2 and self.attacherVehicle and self:getDetachIsPossible() then
			local attacherJointTransform = self:getHydraulicPipeAttacher(self.attacherVehicle);
			if attacherJointTransform then
				g_currentMission:addHelpButtonText(self.hydraulicPipe.manualAttachText[self.hydraulicPipe.manualAttachActive], self.hydraulicPipe.linkInputButton, nil, GS_PRIO_HIGH);
				if InputBinding.hasEvent(self.hydraulicPipe.linkInputButton) then
					if self.hydraulicPipe.manualAttachActive then
						if g_server then
							self:detachPipe();
							g_server:broadcastEvent(HydraulicPipeEvent:new(false, self));
						else
							g_client:getServerConnection():sendEvent(HydraulicPipeEvent:new(false, self));
						end;
					else
						if g_server then
							self:attachPipe(attacherJointTransform);
							g_server:broadcastEvent(HydraulicPipeEvent:new(true, self));
						else
							g_client:getServerConnection():sendEvent(HydraulicPipeEvent:new(true, self));
						end;
					end;
				end;
			end;
		end;
	end;
end;

function HydraulicPipe:updateTick(dt)
end;

function HydraulicPipe:draw()
end;

----------------------------------------------------------------------------------------------------------------

HydraulicPipeEvent = {};
HydraulicPipeEvent_mt = Class(HydraulicPipeEvent, Event);
InitEventClass(HydraulicPipeEvent, "HydraulicPipeEvent");

function HydraulicPipeEvent:emptyNew()
	local self = Event:new(HydraulicPipeEvent_mt);
    return self;
end;

function HydraulicPipeEvent:new(state, object)
	local self = HydraulicPipeEvent:emptyNew();
	self.state = state;
	self.object = object;
	return self;
end;

function HydraulicPipeEvent:readStream(streamId, connection)
	local state = streamReadBool(streamId);
    local object = networkGetObject(streamReadInt32(streamId));
	if state then
		local attacherJointTransform = object:getHydraulicPipeAttacher(object.attacherVehicle);
		object:attachPipe(attacherJointTransform);
	else
		object:detachPipe();
	end;
	if g_server then
		g_server:broadcastEvent(HydraulicPipeEvent:new(state, object));
	end;
end;

function HydraulicPipeEvent:writeStream(streamId, connection)
	streamWriteBool(streamId, self.state);
	streamWriteInt32(streamId, networkGetObjectId(self.object));
end;
